/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */

package java.lang.management;

/**
 * @since 1.5
 */
public enum MemoryType {

    /**
     * Memory on the heap. The heap is the runtime area in the virtual machine,
     * created upon the start-up of the virtual machine, from which memory for
     * instances of types and arrays is allocated. The heap is shared among all
     * threads in the virtual machine.
     */
    HEAP,
    /**
     * Memory that is not on the heap. This encompasses all other storage used
     * by the virtual machine at runtime.
     */
    NON_HEAP;
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        String result = null;
        switch (this) {
        case HEAP:
            result = "Heap memory";
            break;
        case NON_HEAP:
            result = "Non-heap memory";
            break;
        }
        return result;
    }
}
